//
//  OptionsViewController.swift
//  ABC Bluetooth Demo
//
//  Created by AB Circle on 23/11/2020.
//  Copyright © 2020 AB Circle. All rights reserved.
//

import UIKit

/// Delegate protocol for delegate classes to implement
protocol OptionsViewControllerProtocol : NSObjectProtocol {
    
    /// Call back function definition to implement for handling the closing of the Custom Key view
    /// and retrieving the inputted values.
    ///
    /// - Parameters:
    ///   - useDefaultKey: Flag indicating if default key should be used
    ///   - textViewContent: Custom key (as hexadecimal string)
    ///   - nameFilter: Reader Name filter
    ///   - rssiFilter: rssiFilter value (in dbM)
    func onCloseOptionsWindow(useDefaultKey: Bool, textViewContent: String, nameFilter: String, rssiFilter: Int)
}

/// Controller for handling the Custom Key input, readername filter and RSSI filter
class OptionsViewController: UIViewController {
    
    /// :nodoc: View controller identifier
    static let identifier   = "OptionsViewController"
    
    /// delegate, set this to the class acting as delegate for this class using the CustomKeyViewControllerProtocol
    private weak var delegate: OptionsViewControllerProtocol?
    
    /// Flag indicating if default key should be used
    private var useDefaultKey:  Bool = true
    /// Custom key
    private var customKey:      String = ""
    /// Name Filter
    private var nameFilter:     String = ""
    /// RSSI Filter
    private var rssiFilter:     Int    = 0
    
    /// Control outlet for Key Type UISegmentedControl
    @IBOutlet weak var segmentedKeyType: UISegmentedControl!
    /// Control outlet for Custom Key HexEditUITextView
    @IBOutlet weak var customKeyTextView: HexEditUITextView!
    /// Control outlet for Name Filter UITextField
    @IBOutlet weak var nameFilterTextField: UITextField!
    /// Control outlet for RSSI Filter UITextField
    @IBOutlet weak var rssiFilterTextField: NumericUITextField!
    
    /// Called after the controller's view is loaded into memory.
    /// Initialises the local variables, the controls and starts the terminal list updater
    override func viewDidLoad() {
        super.viewDidLoad()
        
        initialiseTextView(textView: customKeyTextView)
        
        segmentedKeyType.selectedSegmentIndex = (useDefaultKey ? 0 : 1)
        customKeyTextView.text = customKey
        nameFilterTextField.text = nameFilter
        rssiFilterTextField.text = String(rssiFilter, radix: 10)

        enableUIFields()
    }
    
    /// Sets the initial values of the view
    ///
    /// - Parameters:
    ///   - parentViewController: Parent controller to use as delegate
    ///   - useDefaultKey: Initial flag indicating if default key is used
    ///   - customKey: Initial custom key
    func setInitialValues(parentViewController: UIViewController, useDefaultKey: Bool, customKey: String, nameFilter: String, rssiFilter: Int) {
        self.delegate       = parentViewController as? OptionsViewControllerProtocol
        self.useDefaultKey  = useDefaultKey
        self.customKey      = customKey
        self.nameFilter     = nameFilter
        self.rssiFilter     = rssiFilter
    }
    
    /// Handles key type change
    ///
    /// - Parameter sender: UI control
    @IBAction func onKeyTypeChanged(_ sender: UISegmentedControl) {
        useDefaultKey = (sender.selectedSegmentIndex == 0)
        enableUIFields()
    }
    
    /// OK button handler
    ///
    /// - Parameter sender: UI control
    @IBAction func onOKClicked(_ sender: UIButton) {
        if let rssi = rssiFilterTextField.text {
            rssiFilter = (Int(rssi) ?? 0)
        }
        else {
            rssiFilter = 0
        }
        if useDefaultKey {
            
            delegate?.onCloseOptionsWindow(useDefaultKey: useDefaultKey,
                                             textViewContent: customKeyTextView.text,
                                             nameFilter: nameFilterTextField.text ?? "",
                                             rssiFilter: rssiFilter)
            closeThis()
        }
        else {
            if Helper.hexStringToData(hexString: customKeyTextView.text).count == 16 {
                delegate?.onCloseOptionsWindow(useDefaultKey: useDefaultKey,
                                                 textViewContent: customKeyTextView.text,
                                                 nameFilter: nameFilterTextField.text ?? "",
                                                 rssiFilter: rssiFilter)
                closeThis()
            }
            else {
                let alert = UIAlertController(title: "Warning", message: "Custom key must be 16 bytes long", preferredStyle: .alert)
                alert.addAction(UIAlertAction(title: NSLocalizedString("OK", comment: "Default action"), style: .default, handler: nil))
                self.present(alert, animated: true, completion: nil)
            }
        }
    }
    
    /// Close button handler
    ///
    /// - Parameter sender: UI control
    @IBAction func onCloseClicked(_ sender: UIButton) {
        closeThis()
    }
    
    /// Initialises the text view properties
    ///
    /// - Parameter textView: Textview control to initialise
    private func initialiseTextView(textView: UITextView)
    {
        textView.autocapitalizationType = .allCharacters
        textView.text                   = ""
        textView.layer.borderWidth      = 1.0
        textView.layer.cornerRadius     = 8
        textView.layer.borderColor      =  UIColor.lightGray.cgColor
    }
    
    /// Enable or disable UI
    private func enableUIFields() {
        if useDefaultKey {
            customKeyTextView.isEditable    = false
        }
        else {
            customKeyTextView.isEditable    = true
        }
    }
    
    /// Closes this view
    private func closeThis() {
        self.dismiss(animated: true, completion: nil)
    }
}
