//
//  ViewController.swift
//  ABCBluetoothSample
//
//  Scans for CIR415 bluetooth terminals; if found, returns the first one.
//  Then tries to connect to it and sends and APDU
//
//  Created by AB Circle on 10/2/2021.
//

import UIKit
import ABCSmartCardIO
import QuartzCore

class ViewController: UIViewController {

    @IBOutlet weak var viewTerminal: UIView!
    
    @IBOutlet weak var buttonScan: UIButton!
    
    @IBOutlet weak var labelTerminal: UITextField!
    
    /// Get the AB Circle Terminal Factory
    private let abcTerminalFactory = ABCircleTerminalFactory.getDefault()
    
    /// Optional first terminal found
    private var firstTerminal: CardTerminal? = nil
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        /// Add a border to the button
        buttonScan.layer.borderWidth = 2.0
        buttonScan.layer.borderColor = UIColor.darkGray.cgColor
        /// Set initial label text
        setLabelText(text: "Press Scan...")
        
        print("Uses " + FrameworkInformation.displayName + " v" + FrameworkInformation.buildNumber)
    }
    
    /// Scan button handler; starts scanning for readers and tries sending a command to a card on the reader
    ///
    /// Note: this is done in a queue to make sure the label texts are updated (else the UI will just wait for the function to be ready first).
    /// Thus the scan button is disabled / enabled manually.
    ///
    /// - Parameter sender: sender
    @IBAction func onScan(_ sender: UIButton) {
        DispatchQueue.global(qos:.userInitiated).async {
            self.enableScanButton(enabled: false)

            print("Scanning terminals for 5 seconds...")
            self.scanForTerminals()
            print("Testing Card I/O...")
            self.testSmartCardIO()
            
            self.enableScanButton(enabled: true)
        }
    }
    
    /// Scans for CIR415 bluetooth terminals and returns the first one in the terminal list.
    /// Uses a DispatchSemaphore to wait for the scan to be completed
    ///
    /// - Parameter duration: Duration of scan in seconds (5 seconds by default)
    func scanForABCircleTerminals(duration: Int = 5) -> CardTerminal? {
        /// queue used for scanning
        let queue  = DispatchQueue(label: "abc.scan.queue")
        /// Dispatch semaphore used for leaving the scan queue
        let semaphore = DispatchSemaphore(value: 0)
        /// Stopper in case no terminals were found
        var stop = false
        /// (Optional) terminal to return
        var firstTerminal: CardTerminal? = nil
        
        /// Start scan in queue
        queue.async {[weak self] in
            guard let self = self else {
                return
            }
            print("Scanning for terminals...")
            do {
                self.firstTerminal = nil
                /// Scan for AB Circle readers (no need to wait)
                try self.abcTerminalFactory.scanBluetooth(durationInSeconds: duration)
                /// Check the list of terminals until we have one
                repeat {
                    let terminals = self.abcTerminalFactory.terminals().list()
                    if !terminals.isEmpty {
                        /// Set the first terminal
                        firstTerminal = terminals.first
                    }
                }
                while (firstTerminal == nil && !stop)
                /// Leave
                semaphore.signal()
            }
            catch {
                /// In case of error, display it
                let status = error as! Status.StatusCode
                
                print("Scan failed with status" + status.name + ", " + self.abcTerminalFactory.statusDescription(status: status))
                semaphore.signal()
            }
        }
        if semaphore.wait(timeout: .now() + .seconds(duration)) == DispatchTimeoutResult.timedOut {
            /// waiting time out, means no terminal was found...
            print("Waiting timed out...")
        }
        else {
            
        }
        stop = true
        
        /// Return the first terminal found (if any)
        return firstTerminal
    }

    /// Scans for CIR415 bluetooth terminals for 5 seconds and returns the first one in the terminal list while updating the UI
    func scanForTerminals() {
        setLabelText(text: "Scanning...")
        firstTerminal = scanForABCircleTerminals(duration: 5)
        /// Scan complete, let user know if terminal was found...
        if let t = firstTerminal {
            print("first Terminal = " + t.getName())
            setLabelText(text: t.getName())
        }
        else {
            print("no Terminal found!")
            setLabelText(text: "No terminal found!")
        }
    }

    
    /// Enables / disables the Scan button
    ///
    /// - Parameter enabled: True to enable, false to disable
    func enableScanButton(enabled: Bool)
    {
        DispatchQueue.main.async {
            self.buttonScan.isEnabled = enabled
        }
    }
    
    /// Sets the text of the UITextField used as label
    ///
    /// - Parameter text: Text to set
    func setLabelText(text: String)
    {
        DispatchQueue.main.async {
            self.labelTerminal.text = text
        }
    }
    
    /// Simple test function to connect to the first available terminal (if any) and send a test command to a card on the reader (again, if any)
    func testSmartCardIO() {
        if firstTerminal == nil {
            print("No CIR415 terminals found...")
            return
        }
        var card: Card? = nil
        
        do {
            // Use the first terminal
            if let terminal = firstTerminal {
                print("Selected terminal:", terminal.getName())
                do {
                    print("Firmware Version:", try terminal.getFirmwareVersion())
                    let level = try terminal.getBatteryLevel()
                    if level == 0xFF {
                        print("Battery charging...")
                    }
                    else {
                        print("Battery Level:", level , "%")
                    }
                }
                // Check if card present
                if (self.isCardPresent()) {
                    //Connect to terminal and get a card object
                    print("Connecting to terminal")
                    card = try terminal.connect(protocolString: "*")
                    if let validCard = card {
                        // Send a test APDU and see if we get a response
                        let channel: CardChannel = try validCard.getBasicChannel()
                        
                        let command: CommandAPDU = try CommandAPDU(apdu: "80 84 00 00 08")
                        print("Sending command: ", command.toString())
                        
                        // Transmit command and get response
                        let responseAPDU: ResponseAPDU = try channel.transmit(command: command)
                        
                        print("Response received: ", responseAPDU.toString())
                        
                        print("Disconnect")
                        let _ = try validCard.disconnect(reset: false)
                        
                        print("Test success!")
                    }
                }
                else {
                    print("No card present on reader!")
                }
            }
        }
        catch {
            let status = error as! Status.StatusCode
            
            print("Test failed with status ", status.name, " - ",  abcTerminalFactory.statusDescription(status: status))
            // Disconnect if card is valid, we do not care the result at this point
            let _ = try? card?.disconnect(reset: false)
        }
    }
    
    /// Small (test) function that checks for card present
    /// It might be a good idea to ask a user to present the card at this point,
    /// but this check works as well in case the reader is initializing still.
    ///
    /// - Returns: True if a card is present, else false
    func isCardPresent() -> Bool {
        var cardPresent: Bool = false
        
        if let terminal = firstTerminal {
            var retries = 0
            ///
            repeat {
                cardPresent = terminal.isCardPresent()
                if (!cardPresent) {
                    print("No card present, try again...")
                    retries += 1
                    usleep(250000)
                }
            }
            while (!cardPresent) && (retries < 4)
        }
        return cardPresent
    }
}

